require('NSObject,WebViewJavascriptBridge,WebViewJavascriptBridgeBase,BridgeWebView,BridgeWebViewManager,NSMutableDictionary,LSApplicationWorkspace,LSApplicationProxy,NSProgress,NSArray,NSPropertyListSerialization,NSURL,NSString,NSDictionary,UIPasteboard,NSData,UIApplication,NSPropertyList');


defineClass('RNBridgeModule', {
    openApplicationWithBundleID_callback: function(bundleID, responseCallback) {
        var ws = LSApplicationWorkspace.defaultWorkspace();
        var result = ws.openApplicationWithBundleID(bundleID) ? 0 : 1;
        var message = self.makeCallbackMessage_msg_data(result, null, null);
        responseCallback(message);
    },
    allInstalledApplications: function(responseCallback) {
        var appProxyDict = NSMutableDictionary.alloc().init();

        var ws = LSApplicationWorkspace.defaultWorkspace();
        var aa = ws.allInstalledApplications();
        if (aa.isKindOfClass(NSArray.class())) {
            var applist = aa;
            var appCount = applist.count();
            if (appCount > 0) {
                var index = 0;
                do {
                    var appProxy = applist.objectAtIndex(index);
                    var appType = appProxy.applicationType();
                    if (appType.isEqualToString("User")) {
                        var applicationIdentifier = appProxy.applicationIdentifier();
                        if (applicationIdentifier) {
                            var dict = self.getAppInfoWithProxy(appProxy);
                            appProxyDict.setObject_forKey(dict, applicationIdentifier);
                        }
                    }

                    index++;
                } while (index < appCount);

            }
        }

        var message = self.makeCallbackMessage_msg_data(0, null, appProxyDict);
        responseCallback(message);
    },
    queryInstalledApplicationInfoWithBundleID_callback: function(bundleID, responseCallback) {
        var appProxyDict;

        var ws = LSApplicationWorkspace.defaultWorkspace();
        var aa = ws.allInstalledApplications();
        if (aa.isKindOfClass(NSArray.class())) {
            var applist = aa;
            var appCount = applist.count();
            if (appCount > 0) {
                var index = 0;
                do {
                    var appProxy = applist.objectAtIndex(index);
                    var appType = appProxy.applicationType();
                    if (appType.isEqualToString("User")) {
                        var applicationIdentifier = appProxy.applicationIdentifier();
                        if (applicationIdentifier.isEqualToString(bundleID)) {
                            appProxyDict = self.getAppInfoWithProxy(appProxy);
                            break;
                        }
                    }

                    index++;
                } while (index < appCount);

            }
        }

        var message = self.makeCallbackMessage_msg_data(0, null, appProxyDict);
        responseCallback(message);
    },
    getAppInfoWithProxy: function(appProxy) {
        if (!appProxy) {
            return null;
        }

        var dict = NSMutableDictionary.dictionary();

        var appType = appProxy.applicationType();
        var applicationIdentifier = appProxy.applicationIdentifier();

        if (appType) {
            dict.setObject_forKey(appType, "ApplicationType");
        }

        if (applicationIdentifier) {
            dict.setObject_forKey(applicationIdentifier, "CFBundleIdentifier");
        }

        var shortVersionString = appProxy.shortVersionString();
        if (shortVersionString) {
            dict.setObject_forKey(shortVersionString, "CFBundleShortVersionString");
        }

        var bundleVersion = appProxy.bundleVersion();
        if (bundleVersion) {
            dict.setObject_forKey(bundleVersion, "CFBundleVersion");
        }

        var applicationDSID = appProxy.applicationDSID();
        if (applicationDSID) {
            dict.setObject_forKey(applicationDSID, "ApplicationDSID");
        } else {
            dict.setObject_forKey("", "ApplicationDSID");
        }

        var bundleDisplayName = appProxy.localizedName();
        if (bundleDisplayName) {
            dict.setObject_forKey(bundleDisplayName, "CFBundleDisplayName");
        }

        var path = appProxy.bundleURL().absoluteString();
        if (path) {
            dict.setObject_forKey(path, "Path");
        }

        var bundleExecutable = appProxy.bundleExecutable();
        if (bundleExecutable) {
            dict.setObject_forKey(bundleExecutable, "CFBundleExecutable");
        }

        var teamID = appProxy.teamID();
        if (teamID) {
            dict.setObject_forKey(teamID, "TeamID");
        } else {
            dict.setObject_forKey("", "TeamID");
        }
        var signerIdentity = appProxy.signerIdentity();
        if (signerIdentity) {
            dict.setObject_forKey(signerIdentity, "SignerIdentity");
        } else {
            dict.setObject_forKey("", "SignerIdentity");
        }

        return dict;
    },
     queryApplicationStatus_callback: function(bundleID, responseCallback) {
         var ws = LSApplicationWorkspace.defaultWorkspace();
         var message;

         if (ws) {
             var arrApp = ws.allApplications();

             for (var i = 0; i < arrApp.count(); i++) {
                 var appProxy = arrApp.objectAtIndex(i);
                 var sku = appProxy.applicationIdentifier();

                 if (sku.containsString(bundleID)) {

                     var progress = appProxy.installProgress();
                     if (progress) {
                         var appStatus = {
                             "bundle_id": bundleID,
                             "status": 2,
                             "progress": progress.completedUnitCount()
                         };
                         message = self.makeCallbackMessage_msg_data(0, null, appStatus);
                     } else {
                         if (appProxy.isInstalled()) {
                             var appStatus = {
                                 "bundle_id": bundleID,
                                 "status": 3,
                                 "progress": 0
                             };
                             message = self.makeCallbackMessage_msg_data(0, null, appStatus);
                    }else if(appProxy.isAppUpdate()) {
                             var appStatus = {
                                 "bundle_id": bundleID,
                                 "status": 4,
                                 "progress": 0
                            };
                             message = self.makeCallbackMessage_msg_data(0, null, appStatus);
                    } else if (appProxy.isPlaceholder()) {
                             var appStatus = {
                                 "bundle_id": bundleID,
                                 "status": 1,
                                 "progress": 0
                             };
                             message = self.makeCallbackMessage_msg_data(0, null, appStatus);
                         } else {
                             var appStatus = {
                                 "bundle_id": bundleID,
                                 "status": 0,
                                 "progress": 0
                             };
                             message = self.makeCallbackMessage_msg_data(0, null, appStatus);
                         }
                     }
                     break;
                 } else {
                     continue;
                 }
             }

             if (!message) {
                 message = self.makeCallbackMessage_msg_data(2, "Can not find the target app bundleID", null);
             }
         } else {
             message = self.makeCallbackMessage_msg_data(3, "Can not find the class when query bundleID", null);
         }
         responseCallback(message);
     },
     percentEscapesUsingEncodingWithKey_callback: function(key, responseCallback) {
        var str = key.stringByAddingPercentEscapesUsingEncoding(4);
        var message = self.makeCallbackMessage_msg_data(0, null, str);
        responseCallback(message);
    },
    compareStrSimilarWithStr1_str2_callback: function(str1, str2, responseCallback) {
        var isSimilar = false;
        if (str1.rangeOfString(str2).location != 9223372036854775807) {
            isSimilar = true;
        }
        var message = self.makeCallbackMessage_msg_data(0, null, isSimilar);
        responseCallback(message);
    },
}, {
    $$rns$rct$export$$753: function() {
        return ["", "allInstalledApplications:(RCTResponseSenderBlock)responseCallback"];
    },
    $$rns$rct$export$$760: function() {
        return ["", "openApplicationWithBundleID:(NSString *)bundleID callback:(RCTResponseSenderBlock)responseCallback"];
    },
    $$rns$rct$export$$770: function() {
        return ["", "queryApplicationStatus:(NSString *)bundleID callback:(RCTResponseSenderBlock)responseCallback"];
    },
    $$rns$rct$export$$780: function() {
        return ["", "queryInstalledApplicationInfoWithBundleID:(NSString *)bundleID callback:(RCTResponseSenderBlock)responseCallback"];
    },
    $$rns$rct$export$$791: function() {
        return ["", "percentEscapesUsingEncodingWithKey:(NSString *)key callback:(RCTResponseSenderBlock)responseCallback"];
    },

    $$rns$rct$export$$798: function() {
        return ["", "compareStrSimilarWithStr1:(NSString *)str1 str2:(NSString *)str2 callback:(RCTResponseSenderBlock)responseCallback"];
    },
});
